@@ -90,7 +90,7 @@ static void
 membuf_append(struct membuffer* buf, const char* data, MD_SIZE size)
 {
     if(buf->asize < buf->size + size)
-        membuf_grow(buf, (buf->size + size) * 2);
+        membuf_grow(buf, buf->size + buf->size / 2 + size);
     memcpy(buf->data + buf->size, data, size);
     buf->size += size;
 }
@@ -120,7 +120,7 @@ process_file(FILE* in, FILE* out)
     /* Read the input file into a buffer. */
     while(1) {
         if(buf_in.size >= buf_in.asize)
-            membuf_grow(&buf_in, 2 * buf_in.asize);
+            membuf_grow(&buf_in, buf_in.asize + buf_in.asize / 2);
 
         n = fread(buf_in.data + buf_in.size, 1, buf_in.asize - buf_in.size, in);
         if(n == 0)
@@ -193,24 +193,29 @@ static const option cmdline_options[] = {
     { "stat",                       's', 's', OPTION_ARG_NONE },
     { "help",                       'h', 'h', OPTION_ARG_NONE },
     { "version",                    'v', 'v', OPTION_ARG_NONE },
+
     { "commonmark",                  0,  'c', OPTION_ARG_NONE },
     { "github",                      0,  'g', OPTION_ARG_NONE },
-    { "fverbatim-entities",          0,  'E', OPTION_ARG_NONE },
+
+    { "fcollapse-whitespace",        0,  'W', OPTION_ARG_NONE },
+    { "flatex-math",                 0,  'L', OPTION_ARG_NONE },
     { "fpermissive-atx-headers",     0,  'A', OPTION_ARG_NONE },
+    { "fpermissive-autolinks",       0,  'V', OPTION_ARG_NONE },
+    { "fpermissive-email-autolinks", 0,  '@', OPTION_ARG_NONE },
     { "fpermissive-url-autolinks",   0,  'U', OPTION_ARG_NONE },
     { "fpermissive-www-autolinks",   0,  '.', OPTION_ARG_NONE },
-    { "fpermissive-email-autolinks", 0,  '@', OPTION_ARG_NONE },
-    { "fpermissive-autolinks",       0,  'V', OPTION_ARG_NONE },
-    { "fno-indented-code",           0,  'I', OPTION_ARG_NONE },
+    { "fstrikethrough",              0,  'S', OPTION_ARG_NONE },
+    { "ftables",                     0,  'T', OPTION_ARG_NONE },
+    { "ftasklists",                  0,  'X', OPTION_ARG_NONE },
+    { "funderline",                  0,  '_', OPTION_ARG_NONE },
+    { "fverbatim-entities",          0,  'E', OPTION_ARG_NONE },
+    { "fwiki-links",                 0,  'K', OPTION_ARG_NONE },
+
     { "fno-html-blocks",             0,  'F', OPTION_ARG_NONE },
     { "fno-html-spans",              0,  'G', OPTION_ARG_NONE },
     { "fno-html",                    0,  'H', OPTION_ARG_NONE },
-    { "fcollapse-whitespace",        0,  'W', OPTION_ARG_NONE },
-    { "ftables",                     0,  'T', OPTION_ARG_NONE },
-    { "fstrikethrough",              0,  'S', OPTION_ARG_NONE },
-    { "flatex-math",                 0,  'L', OPTION_ARG_NONE },
-    { "fwiki-links",                 0,  'K', OPTION_ARG_NONE },
-    { "ftasklists",                  0,  'X', OPTION_ARG_NONE },
+    { "fno-indented-code",           0,  'I', OPTION_ARG_NONE },
+
     { 0 }
 };
 
@@ -229,15 +234,14 @@ usage(void)
         "  -v, --version        Display version and exit\n"
         "\n"
         "Markdown dialect options:\n"
-        "(note these are equivalent to some combinations of flags below)\n"
+        "(note these are equivalent to some combinations of the flags below)\n"
         "      --commonmark     CommonMark (this is default)\n"
         "      --github         Github Flavored Markdown\n"
         "\n"
         "Markdown extension options:\n"
         "      --fcollapse-whitespace\n"
         "                       Collapse non-trivial whitespace\n"
-        "      --fverbatim-entities\n"
-        "                       Do not translate entities\n"
+        "      --flatex-math    Enable LaTeX style mathematics spans\n"
         "      --fpermissive-atx-headers\n"
         "                       Allow ATX headers without delimiting space\n"
         "      --fpermissive-url-autolinks\n"
@@ -249,18 +253,25 @@ usage(void)
         "      --fpermissive-autolinks\n"
         "                       Same as --fpermissive-url-autolinks --fpermissive-www-autolinks\n"
         "                       --fpermissive-email-autolinks\n"
-        "      --fno-indented-code\n"
-        "                       Disable indented code blocks\n"
+        "      --fstrikethrough Enable strike-through spans\n"
+        "      --ftables        Enable tables\n"
+        "      --ftasklists     Enable task lists\n"
+        "      --funderline     Enable underline spans\n"
+        "      --fwiki-links    Enable wiki links\n"
+        "\n"
+        "Markdown suppression options:\n"
         "      --fno-html-blocks\n"
         "                       Disable raw HTML blocks\n"
         "      --fno-html-spans\n"
         "                       Disable raw HTML spans\n"
         "      --fno-html       Same as --fno-html-blocks --fno-html-spans\n"
-        "      --ftables        Enable tables\n"
-        "      --fstrikethrough Enable strikethrough spans\n"
-        "      --flatex-math    Enable LaTeX style mathematics spans (e.g. $a+b=c$ becomes <equation>a+b=c</equation>)\n"
-        "      --fwiki-links    Enable wiki links\n"
-        "      --ftasklists     Enable task lists\n"
+        "      --fno-indented-code\n"
+        "                       Disable indented code blocks\n"
+        "\n"
+        "HTML generator options:\n"
+        "      --fverbatim-entities\n"
+        "                       Do not translate entities\n"
+        "\n"
     );
 }
 
@@ -311,6 +322,7 @@ cmdline_callback(int opt, char const* value, void* data)
         case 'L':   parser_flags |= MD_FLAG_LATEXMATHSPANS; break;
         case 'K':   parser_flags |= MD_FLAG_WIKILINKS; break;
         case 'X':   parser_flags |= MD_FLAG_TASKLISTS; break;
+        case '_':   parser_flags |= MD_FLAG_UNDERLINE; break;
 
         default:
             fprintf(stderr, "Illegal option: %s\n", value);
@@ -344,7 +356,7 @@ main(int argc, char** argv)
     if(output_path != NULL && strcmp(output_path, "-") != 0) {
         out = fopen(output_path, "wt");
         if(out == NULL) {
-            fprintf(stderr, "Cannot open %s.\n", input_path);
+            fprintf(stderr, "Cannot open %s.\n", output_path);
             exit(1);
         }
     }
